'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: UDS_UserFolders_Provision.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 11/12/2007
'
' USAGE:  
' cscript.exe UDS_UserFolders_Provision.vbs UserFolderPath
'
' Creates a physical namespace for user data stores.
' See the Windows Administration Resource Kit for documentation
' See the scripts UDS_UserFolders_Provision_DFS And
' UDS_UserFolders_Extended_Provision_DFS for more advanced solutions.
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

Option Explicit

Dim FSO, WSHShell
Dim sParentFolder, sUsername, sCommand, sFolder
Dim arr, i, ret
Dim sDomainNetBIOS, sTemplate_UserData
Dim bInheritanceError

' CONFIGURATION BLOCK
' Your domain name
sDomainNetBIOS = "CONTOSO"
' The name of the quota template you want applied to the user's Data folder
' Change to blank ("") if you want to skip the application of a quota
sTemplate_UserData = "1 GB User Data Storage Limit with 100 MB Extension"

Create_Common_Objects
Check_Arguments

If Not FolderPath_Create(sParentFolder) Then
	WScript.Echo sParentFolder & " could not be found and could not be created."
	WScript.Quit
End If

' Determine the user name by looking at the last folder in the folder path
sUsername = FSO.GetFolder(sParentFolder).Name

sFolder = sParentFolder & "\Data"
If FolderPath_Create( sFolder ) Then
	' Management logic here:
	' any code you want to perform after the folder has beeen created
	' For example, apply a quota:
	If sTemplate_UserData > "" Then
		sCommand = "dirquota quota add /path:""" & sFolder & _
			""" /sourcetemplate:""" & sTemplate_UserData & """"
		arr = Execute_Capture(sCommand, 10, True)
	End If
End If

sFolder = sParentFolder & "\Data\Documents"
If FolderPath_Create( sFolder ) Then
	' Management logic here:
	' any code you want to perform after the folder has beeen created
End If

sFolder = sParentFolder & "\Data\Desktop"
If FolderPath_Create( sFolder ) Then
	' Management logic here:
	' any code you want to perform after the folder has beeen created
End If

sFolder = sParentFolder & "\Backups"
If FolderPath_Create( sFolder ) Then
	' Management logic here:
	' any code you want to perform after the folder has beeen created
End If

sFolder = sParentFolder & "\Profile"
If FolderPath_Create( sFolder ) Then
	' Management logic here:
	' any code you want to perform after the folder has beeen created
End If

sFolder = sParentFolder & "\Profile.V2"
If FolderPath_Create( sFolder ) Then
	' Management logic here:
	' any code you want to perform after the folder has beeen created
End If

' INHERITANCE
' Ensure that inheritance is enabled for the %username% folder and the profile folders
' If the roaming profile created the folder first, it will be off!
' WS2008 code using icacls.exe with the new /inheritance switch
sCommand = "icacls """ & sParentFolder & """ /inheritance:e"
arr = Execute_Capture(sCommand, 10, True)
if arr(0) > 0 then bInheritanceError = True
sCommand = "icacls """ & sParentFolder & "\Profile"" /inheritance:e"
arr = Execute_Capture(sCommand, 10, True)
If arr(0) > 0 then bInheritanceError = True
sCommand = "icacls """ & sParentFolder & "\Profile.V2"" /inheritance:e"
arr = Execute_Capture(sCommand, 10, True)
if arr(0) > 0 then bInheritanceError = True

If bInheritanceError Then
	WScript.Echo "There was an error enabling inheritance using icacls.exe." & VbCrLf & VbCrLf & _
		"If you are executing this script from a system that is not running" & VbCrLf & _
		"Windows Server 2008 or Windows Vista SP1, then the error is expected: " & VbCrLf & _
		"older versions of icacls.exe do not support the /inheritance switch." & VbCrLf & VbCrLf & _
		"Once the script is complete, you must check to ensure inheritance is enabled" &VbCrLf & _
		"on the following folders: " & VbCrLf & _
		"    - " & sParentFolder & VbCrLf & _
		"    - " & sParentFolder & "\Profile" & VbCrLf & _
		"    - " & sParentFolder & "\Profile.V2" & VbCrLf & VbCrLf & _ 
	WScript.Echo
End If

' We've been creating all of these folders.  We need to ensure the user has full control and owns them.
sCommand = "icacls """ & sParentFolder & """ /grant " & sDomainNetBIOS &"\" & sUsername & ":(CI)(OI)F"
arr = Execute_Capture(sCommand, 10, True)
sCommand = "icacls """ & sParentFolder & """ /setowner " & sDomainNetBIOS &"\" & sUsername & " /T"
arr = Execute_Capture(sCommand, 10, True)

WScript.Echo sParentFolder & " provisioned."

Sub Create_Common_Objects()
	Set FSO = CreateObject("Scripting.FileSystemObject")
	Set WSHShell = CreateObject("Wscript.Shell")
End Sub

Sub Check_Arguments()
	On Error Resume Next
	If Right(WScript.Arguments(0),1) = "?" Then
		Call Usage()
		WScript.Quit
	End If
	sParentFolder = WScript.Arguments(0)
	If sParentFolder = "" Then 
		WScript.Echo "No parent folder provided"
		WScript.Echo
		Call Usage()
		WScript.Quit
	End If
	On Error GoTo 0
End Sub

Sub Usage()
		WScript.Echo "UDS_UserFolders_Provision"
		WScript.Echo "========================="
		WScript.Echo "Create user data stores for a user."
		WScript.Echo 
		WScript.Echo "USAGE:"
		WScript.Echo "   cscript.exe UDS_UserFolders_Provision.vbs UserFolderPath"
		WScript.Echo
		WScript.Echo "WHERE:"
		WScript.Echo
		WScript.Echo "UserFolderPath"
		WScript.Echo "--------------"
		WScript.Echo "Full local or network path to a user's folder,"
		WScript.Echo "e.g. E:\Users\jfine or \\server01\users$\jfine."
		WScript.Echo "The user's folder will be created if it does not already exist."
		WScript.Echo "It is assumed that the last folder in the path (e.g. jfine) is the user's"
		WScript.Echo "pre-Windows 2000 Logon Name. That account must already exist. It is given"
		WScript.Echo "ownership and Full Control of the folder"
		WScript.Echo
		WScript.Echo "Run the script from a Windows Vista SP1 or Windows Server 2008 system."
		WScript.Echo "Run locally on a Windows Server 2008 server as an adminsitrator."
		WScript.Echo "Run remotely gainst a Windows Server 2008 or Windows Server 2003 system"
		WScript.Echo "as a user with sufficient NTFS permissions to create the folders,"
		WScript.Echo "DFS namespace delegation to create the DFS namespace, and the"
		WScript.Echo "Restore Files And Directories user right to transfer ownership."
End Sub

Function FolderPath_Create(FolderPath)
	' VERSION 070918
	' Creates a folder path specified by FolderPath
	' and returns True (success) or False (failure)
	'
	' Recursively create a folder path
	' This is an "inside out" recursion that starts with the lowest level,
	' works its way "up" to the nearest ancestor that exists, then
	' creates folders down from there as it works its way out of recursions
	'
	' INPUT:	FolderPath:			desired folder path
	' OUTPUT:	FolderPath_Create:	TRUE: 	success
	'								FALSE:	failure
	
	Dim bFolderExists
	
	FolderPath_Create = False
	On Error Resume Next
	bFolderExists = FSO.FolderExists(FolderPath)
	If Err.Number <> 0 Then
		' This handles scenarios where a sharename in a UNC doesn't exist.
		' I was getting Out of Memory errors on the FolderExists method.
		FolderPath_Create = False
		Exit Function
	End If
	If Not bFolderExists Then
		' If the folder doesn't exist already, test its parent
		' (RECURSIVE CALL TO SELF)
		If FolderPath_Create(FSO.GetParentFolderName(FolderPath)) Then
			' Create the folder
			FolderPath_Create = True
			Call FSO.CreateFolder(FolderPath)
		End If
	Else
		' This level of folder does exist
		FolderPath_Create = True
	End If
End Function

Function Execute_Capture(ByVal sCommand, ByVal iTimeout, ByVal bTerminate)
	' VERSION 071112
	' Executes command in sCommand
	' Times out after iTimeout and (optionally) terminates the process
	' Returns exit code, StdOut & StdErr
	'
	' INPUTS:	sCommand: 			Command to run
	'			iTimeout:			Time (in seconds) to wait for process
	'			bTerminate:			TRUE  = terminate process after timeout
	'								FALSE = return from function
	' RETURNS:	Execute_Capture:	*ARRAY*
	'				(0): 			Exit code of command 
	'								0.01 = timed out but NOT terminated
	'								0.99 = timed out and TERMINATED
	'				(1):			StdOut
	'				(2):			StdErr
	
	Dim oExec, iTimer, iExitCode, sOut, sErr

	On Error Resume Next
	Set oExec = WSHShell.Exec(sCommand)
	If Err.Number<>0 Then
		' Could not successfully run command
		iExitCode = 500
		sOut = ""
		sErr = "Could not execute command: " & VbCrLf & _
			sCommand & VbCrLf & Err.Description
		Execute_Capture = Array(iExitCode, sOut, sErr)
		Exit Function
	End If
	On Error GoTo 0
	
	iTimer = 0
	Do While oExec.Status = 0
		WScript.Sleep 100
		iTimer = iTimer + 100
		If (iTimer/1000) > iTimeout Then Exit Do
	Loop
	
	If oExec.Status = 0 Then
		' Not completed, but timed out
		If bTerminate Then
			' Terminate
			oExec.Terminate
			iExitCode = 0.99
		Else
			' Timeout but do not terminate
			iExitCode = 0.01
		End If
	Else
		' Completed
		iExitCode = oExec.ExitCode
	End If

	sOut = oExec.StdOut.ReadAll
	sErr = oExec.StdErr.ReadAll
	Execute_Capture = Array(iExitCode, sOut, sErr)
	
End Function
