'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Computers_DesktopsLaptops.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 11/1/2007
'
' USAGE:  
' cscript.exe Computers_DesktopsLaptops.vbs
'
' This VBScript code looks for any desktop or laptop computers that Do
' not belong to the Desktops and Laptops groups, and adds them.
' It can be run as a scheduled task to keep those groups "in synch"
' with the computers in Active Directory.
'
' There is no need to worry about "removing" or "sync'ing" members
' in this case because when a computer is removed from AD, it Is
' automatically removed from the group.
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================
Option Explicit 

Const adStateClosed = 0
Dim sDesktopGroupDN, sLaptopGroupDN, sDesktopNameMatch, sLaptopNameMatch, sSearchDN

'==========================================================================
' CONFIGURATION BLOCK
sDesktopNameMatch = "D*"
sDesktopGroupDN = "CN=CMP_Desktops,OU=Systems,OU=Groups,DC=contoso,DC=com"
sLaptopNameMatch = "L*"
sLaptopGroupDN = "CN=CMP_Laptops,OU=Systems,OU=Groups,DC=contoso,DC=com"
sSearchDN = "ou=clients,dc=contoso,dc=com"
'==========================================================================

Dim sLDAPQuery, sSearchScope, sProperties, oComputers
sSearchScope = "subtree"
sProperties = "ADsPath"

sLDAPQuery = "(&(objectCategory=computer)" & _
	"(samAccountName=" & sDesktopNameMatch & ")" & _
	"(!memberOf=" & sDesktopGroupDN & "))"
Set oComputers = AD_Query(sLDAPQuery, sSearchDN, sSearchScope, sProperties)
If oComputers Is Nothing Then
	' No records - all computers are in the group
ElseIf oComputers.BOF And oComputers.EOF Then
	' No records - all computers are in the group
Else
	' Found records
	oComputers.MoveFirst
	Do While Not oComputers.EOF
		Group_AddMember "LDAP://" & sDesktopGroupDN, oComputers.Fields("ADsPath")
		oComputers.MoveNext
	Loop
End If

sLDAPQuery = "(&(objectCategory=computer)" & _
	"(samAccountName=" & sLaptopNameMatch & ")" & _
	"(!memberOf=" & sLaptopGroupDN & "))"
Set oComputers = AD_Query(sLDAPQuery, sSearchDN, sSearchScope, sProperties)
If oComputers Is Nothing Then
	' No records - all computers are in the group
ElseIf oComputers.BOF And oComputers.EOF Then
	' No records - all computers are in the group
Else
	' Found records
	oComputers.MoveFirst
	Do While Not oComputers.EOF
		Group_AddMember "LDAP://" & sLaptopGroupDN, oComputers.Fields("ADsPath")
		oComputers.MoveNext
	Loop
End If

Sub Group_AddMember(ByVal sGroupADsPath, ByVal sMemberADsPath)
	Dim oGroup, oMember
	On Error Resume Next
	Set oGroup = GetObject(sGroupADsPath)
	If Err.Number <> 0 Then
		' Error handling - could not locate group
		' WScript.Echo "Could not locate group: " & sGroupOUADsPath
		Err.Clear
		Exit Sub
	End If
	Set oMember = GetObject(sMemberADsPath)
	On Error Resume Next
	oGroup.Add oMember.ADsPath
	If Err.Number > 0 then
		' Error handling - could not add member to group
		' WScript.Echo "Error adding " & sMemberADsPath & " to " & sGroupADsPath
		' WScript.Echo "     " & Err.Description
		Err.Clear
	Else
		' Success handling - added member to group
		' WScript.Echo "Added " & sMemberADsPath & " to " & sGroupADsPath
	End If
	On Error Goto 0
End Sub

Function AD_Query(ByVal sLDAPQuery, ByVal sSearchDN, ByVal sSearchScope, ByVal sProperties)
	' VERSION 071029
	' Inputs:
	' sLDAPQuery: an LDAP query
	' sSearchDN: the DN within which to search (often, the DN of the domain, e.g. dc=contoso, dc=com)
	' sSearchScope: the AD Search scope, which can be subtree or onelevel or base (subtree most common)
	' sProperties: the attributes to return from matching objects, comma-delimited.
	'				if blank, ADsPath is returned
	' REQUIRES:
	' Const adStateClosed = 0

	Dim oConnection
	Dim oRecordset
	
	If trim(sProperties) = "" then sProperties = "ADsPath"
	
	' Open an ADO connection using null credentials
	Set oConnection = CreateObject("ADODB.Connection")
	oConnection.Provider = "ADsDSOObject"
	'On Error Resume Next
	oConnection.Open "", vbNullString, vbNullString
	If oConnection.State = adStateClosed Then
	     ' Error handling code
	     ' WScript.Echo "ERROR: Connection to AD failed."
	     Exit Function
	End If
	
	' Build the LDAP Query
	sLDAPQuery = "<LDAP://" & sSearchDN & ">;" & _
		sLDAPQuery & ";" & _
		sProperties & ";" & sSearchScope
	' Don't forget the first ampersand as the AND logic
	
	' Retrieve the result set, close the connection, and check to make
	' sure we received at least one result
	' WScript.Echo sLDAPQuery
	Set oRecordset = oConnection.Execute (sLDAPQuery)
	If oRecordset.EOF and oRecordset.BOF Then
		' Error handling code
	    ' WScript.Echo "ERROR: No objects found." & VbCrLf & sLDAPQuery & vbCrLf & VbCrLf
	    Set AD_Query = Nothing
	Else
		Set AD_Query = oRecordset.clone
	End If
	oRecordset.Close
	'oConnection.Close
	
End Function