'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Computer_InfoToDescription.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 11/1/2007
'
' USAGE:  
' cscript.exe Computer_InfoToDescription.vbs
'
' This script runs on a computer and populates the computer's
' DESCRIPTION in Active Directory with information about the
' last user who logged on, and the Active Directory site.
' The COMMENT field is used as a "history" of recent logons.
'
' It can be run as a startup or logon script.
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

'==========================================================================
' Configuration Block
sDomainDN 		= "dc=contoso,dc=com"
'==========================================================================

Set oADSysInfo 	= CreateObject("ADSystemInfo")
sADSite 		= oADSysInfo.SiteName
sLogonSite 		= oADSysInfo.GetDCSiteName(".")
sComputerName 	= GetComputerName
sLastUser 		= GetLastUserLoggedOn(".")

'==========================================================================
' Customize this line to build the description
sDescription = "Last user: " & sLastUser & "  |  Current site: " & sADSite

sComputerADsPath = ADObject_Find_UCG(sComputerName, sDomainDN)
Set oComputer = GetObject(sComputerADsPath)
' Enter the new description
oComputer.Description = sDescription

' Use the Comment field as a Log
' Each new "Description" is added to the end of the Comment field
sComment = oComputer.Comment 
if Len(sComment) = 0 then sComment = sDescription Else sComment = sComment & vbNewLine & sDescription
' To keep the Comment field from getting too long
' we chop off the first (earliest) comment
if Len(sComment)>255 then sComment = Mid(sComment,Instr(sComment,vbNewLine),Len(sCOmment))
oComputer.Comment = sComment

' All done. Commit the changes.
oComputer.SetInfo

WScript.Quit(0)


Function ADObject_Find_UCG(sObjectName, sSearchDN)
	' VERSION 070706
	' Returns the full ADsPath (LDAP://...) of a user, computer, or group
	' Inputs:
	' 		sObjectName: The unique identifier for the object class.  The script supports:
	'			User: 		sAMAccountName (pre-Windows 2000 logon name)
	'			Group: 		sAMAccountName (pre-Windows 2000 logon name)
	'			Computer: 	Name (translated by the script to the computer's sAMAccountName by adding a $)
	' 		sSearchDN: the DN within which to search (often, the DN of the domain, e.g. dc=contoso, dc=com)

	Dim oConnection
	Dim oRecordset
	Dim sLDAPObjectQuery
	Dim sLDAPIdentifierQuery
	Dim sLDAPQuery
	Dim sProperties
	Dim oADObject
	Dim aProperties
	Dim sProperty
	Dim sLDAPIdentifier
	Dim sSearchScope
	
	sLDAPIdentifier = "samAccountName"
	sProperties = "ADsPath"
	sSearchScope = "subtree"
	
	' Open an ADO connection using null credentials
	Set oConnection = CreateObject("ADODB.Connection")
	oConnection.Provider = "ADsDSOObject"
	'On Error Resume Next
	oConnection.Open "", vbNullString, vbNullString
	If oConnection.State = 0 Then ' 0 = adStateClosed
	     ' Error handling code: can't connect to AD
	     ADObject_Find_UCG = ""
	     Exit Function
	End If
	
	' Build the LDAP Query
	sLDAPQuery = "<LDAP://" & sSearchDN & ">;"
	sLDAPQuery = sLDAPQuery & _
		"(|(samAccountName=" & sObjectName & ")" & _
		"(samAccountName=" & sObjectName & "$));" & _
		sProperties & ";" & sSearchScope
	' Retrieve the result set, close the connection, and check to make
	' sure we received at least one result
	Set oRecordset = oConnection.Execute (sLDAPQuery)
	If oRecordset.EOF and oRecordset.BOF Then
		' Error handling code: no object found
	    ADObject_Find_UCG = ""
	     Exit Function
	End If
	
	ADObject_Find_UCG = oRecordset.Fields("ADsPath")
	oRecordset.Close
	oConnection.Close
	
End Function

Function GetComputerName()
    Dim WshNetwork
    Set WshNetwork = WScript.CreateObject("WScript.Network")
    ' default value
    GetComputerName = "."
    GetComputerName = WshNetwork.ComputerName 
End Function

Function GetLastUserLoggedOn(sComputer)
	' Identifies the last user to log on locally to a computer
	' by looking at the DefaultUserName value of the Winlogon registry key
	
	Const HKLM = &H80000002 		' HKEY_LOCAL_MACHINE
	
	On Error Resume Next
	Set oReg = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" _
	       & sComputer & "/root/default:StdRegProv")
	
	If Err.Number <> 0 Then
	   GetLastUserLoggedOn = ""   
	   Err.Clear
	Else
	   sPath = "SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon"
	   sValueName = "DefaultUserName"
	
	   rtn = oReg.GetStringValue(HKLM, sPath, sValueName, sValue)
	   GetLastUserLoggedOn = sValue
	End If
	On Error GoTo 0

End Function