'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: User_Rename.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 12/13/2007
'
' Configure name properties of a user object
' 
' USAGE
' USER_RENAME.vbs /user:USER
'	[/upn:"jdoe@contoso.com"] [/samaccountname:"jdoe"]
'	[/commonname:"John Doe"] [/displayname:"Doe, John"]
' 	[/firstname:John] [/initial:Q] [/middlename:Franklin][/lastname:Doe]
' where USER can be the sAMAccountName, dn, or ADsPath of a user object
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
' You are free to use this script and make derivatives of it,
' but you may not remove any of this header.
' We also ask that any functional improvements be reported to
' Intelliem at the URL above for the benefit of all.
'==========================================================================

Option Explicit

Dim bInteractive
Dim sDomainDNS, sDomainNetBIOS, sDomainDN
Dim sNewSAM, snewUPN, sNewCN, sNewDisplay, sUser
Dim sNewFirst, sNewInit, sNewMiddle, sNewLast

' ============================================================================
' CONFIGURATION BLOCK
bInteractive = True		' TRUE:  Script can be run from command prompt.
						' FALSE: Limits the output of the script
sDomainDNS = "contoso.com"
sDomainDN = "dc=contoso,dc=com"
sDomainNetBIOS = "CONTOSO"
' ============================================================================

Call MainRoutine()	' Code is contained in a subroutine to make it easier to port to an HTA
WScript.Quit

Sub MainRoutine()
	Dim oUser, sUserADsPath, oTargetOU

	Call Arguments()				' Take what was passed to the script
	If Not Validate() Then Exit Sub	' See if the results are valid, and if not, quit
	
	' Check that user exists by connecting to it
	' On Error Resume Next
	Set oUser = ADObject_Get_Generic(sUser, sDomainDN)
	If oUser is Nothing or Err.Number <> 0 Then
		WScript.Echo "Error finding " & sUser
		Err.Clear
		WScript.Quit(500)	' An error must have occurred
	End If
	On Error GoTo 0
	
	' Write properties. Note the last FALSE in each line means that
	' the code will not "clear" an attribute that was 
	' not specified on the command line or was set to blank
	' So a user's name attribute will NOT be erased/cleared by this routine
	WriteProperty oUser, "sAMAccountName", sNewSAM, False, False
	WriteProperty oUser, "userPrincipalName", snewUPN, False, False
	WriteProperty oUser, "displayName", sNewDisplay, False, False
	WriteProperty oUser, "givenName", sNewFirst, False, False
	WriteProperty oUser, "initials", sNewInit, False, False
	WriteProperty oUser, "middleName", sNewMiddle, False, False
	WriteProperty oUser, "sn", sNewLast, False, False
	oUser.SetInfo
	
	' Are we changing the user's CN?
	If sNewCN > "" Then
		'On Error Resume Next
		Set oTargetOU = GetObject(oUser.Parent)
		oTargetOU.MoveHere oUser.AdsPath, "cn=" & sNewCN 
		If Err.number <> vbEmpty Then
			WScript.Echo "** ERROR ** changing the CN of " & oUser.cn & " to " & sNewCN
			WScript.Echo "**          possibly due to a conflict with an existing user with the same common name (CN)."
			Err.Clear
		End If
		On Error Goto 0
	End If
	
	WScript.Quit (0)
		
End Sub

Sub	Arguments()
	Dim oArgs
	
	' Check for "HELP"
	If (WScript.Arguments.Count = 0) Or _
		(WScript.Arguments.Named.Exists("?")) Then 
		Call Usage()
		WScript.Quit()
	End If
	
	Set oArgs = WScript.Arguments.Named
	sUser = oArgs("user")
	snewUPN = oArgs("upn")
	sNewSAM = oArgs("samaccountname")
	sNewCN = oArgs("commonname")
	sNewDisplay = oArgs("displayname")
	sNewFirst = oArgs("firstname")
	sNewInit = oArgs("initial")
	sNewMiddle = oArgs("middlename")
	sNewLast = oArgs("lastname")
End Sub

Function Validate()
	' Business logic: which arguments are required?
	' Here, user must be specified and at least one of these Arguments
	If sUser = "" Or ((snewUPN="") And (sNewSAM="") And (sNewCN="") And (sNewDisplay="") And _
					  (sNewFirst="") And (sNewInit="") And (sNewMiddle="") And (sNewLast="")) Then 
		WScript.Echo "Required argument(s) missing."
		Call Usage()
		Validate = False
	Else
		Validate = True
	End If		
End Function

Sub Usage()
	If bInteractive Then
		WScript.Echo
		WScript.Echo "USER_RENAME.VBS"
		WScript.Echo "Changes several name properties of user objects"
		WScript.Echo "Usage: cscript.exe  " & WScript.ScriptName & " /user:<USER>" & VbCrLf & _
					 "                    [/upn:""jdoe@contoso.com""] [/samaccountname:""jdoe""]" & VbCrLf & _
					 "                    [/commonname:""John Doe""] [/displayname:""Doe, John""]" & VbCrLf & _
					 "                    [/firstname:John] [/initial:Q] [/middlename:Franklin]" & VbCrLf & _
					 "                    [/lastname:Doe]" & VbCrLf & _
					 "where <USER> is the pre-Windows 2000 logon name (sAMAccountName)," & VbCrLf & _
					 "                    distinguishedName, or aDSPath of a user"

	End If
End Sub

Function WriteProperty(ByVal oObject, ByVal sPropName, ByVal vPropValue, ByVal bIsMultivalued, ByVal bDelete)
	' Version 071120
	' Write a single valued property with control over deletion
	'
	' bIsMultivalued		The attribute is multivalued (so that PutEx is used - not implemented yet)
	' bDelete				Deletes an attribute if vPropValue is empty
	
	Const ADS_PROPERTY_CLEAR = 1
	
	If vPropValue > "" Then
		On Error Resume Next
		oObject.Put sPropName,vPropValue
		If Err.Number > 0 Then 
			WriteProperty = False
			Err.Clear 
		Else
			WriteProperty = True
		End If
		On Error Goto 0
	ElseIf bDelete Then
		oObject.PutEx ADS_PROPERTY_CLEAR, sPropName, 0
		WriteProperty = True
	End If
End Function

' #region Active Directory object find routines

Function ADObject_Get_Generic(ByVal sObject, ByVal sSearchDN)
	' Version 071204
	' Takes any input (name, DN, or ADsPath) of a user, computer, or group, And
	' returns an object reference to the object, or nothing if it does not exist
	' This is a wrapper around ADObject_Find_Generic.
	' Requires ADObject_Find_Generic.
	' See notes for ADObject_Find_Generic
	
	Dim oADObject, sADsPath
	sADsPath = ADObject_Find_Generic(sObject, sSearchDN)
	If sADsPath > "" Then
		Set ADObject_Get_Generic = GetObject(sADsPath)
	Else
		Set ADObject_Get_Generic = Nothing
	End If
	
End Function

Function ADObject_Find_Generic(ByVal sObject, ByVal sSearchDN)
	' Version 071130
	' Takes any input (name, DN, or ADsPath) of a user, computer, or group, and
	' returns the ADsPath of the object as a way of validating that the object exists
	'
	' INPUTS:	sObject 				DN or ADsPath to an object
	'					 				sAMAccountName (pre-Windows 2000 logon name) of a user or group
	'					 				computer name of a computer
	'			sSearchDN 				the DN within which to search (often, the DN of the domain, e.g. dc=contoso, dc=com)
	'
	' RETURNS:	ADObject_Find_Generic 	ADsPath (LDAP://...) of the object
	'								   	blank if object was not found
	'
	' NOTES:	ASSUMPTION: computers, users & groups have unique names. See note inline.
	'
	' REQUIRES	AD_Search_Array routine
	'			AD_Search_RS routine
	' 			ADObject_Validate routine
	
	Dim aResults, sLDAPQuery
	Select Case ADObject_NameType(sObject)
		Case ""
			ADObject_Find_Generic = ""
		Case "adspath"
			ADObject_Find_Generic = ADObject_Validate(sObject)
		Case "distinguishedname"
			ADObject_Find_Generic = ADObject_Validate("LDAP://" & sObject)
		Case "name"
				' Assumption: No computer has the same name as a user's or group's sAMAccountName
				' otherwise, this query will return more than one result
				sLDAPQuery = "<LDAP://" & sSearchDN & ">;" & _
							 "(|(samAccountName=" & sObject & ")(samAccountName=" & sObject & "$));" & _
							 "aDSPath;subtree"
				aResults = AD_Search_Array (sLDAPQuery)
				If Ubound(aResults) = -1 Then
					ADObject_Find_Generic = ""
				Else
					ADObject_Find_Generic = aResults(0)
				End If
	End Select
	
End Function

Function ADObject_NameType(ByVal sObjectName)
	' Version 071204
	' Evaluates sObjectName to determine what type of name it is
	' Returns	ADObject_NameType	adspath
	'								distinguishedname
	'								name
	'								blank if sObjectName = ""
	
	Dim sNameType
	
	If Len(sObjectName) = 0 Then
		sNameType = ""

	ElseIf Len(sObjectName) < 3 Then
		' can't be a DN or an ADsPath - must be a name
		sNameType = "name"
	
	ElseIf Ucase(Left(sObjectName,3)) = "CN=" Then
		' is a DN
		sNameType = "distinguishedname"
		
	ElseIf Len(sObjectName) < 8 Then
		' too short to be an ADsPath and isn't a DN, so it must be a name
		sNameType = "name"
		
	ElseIf UCase(Left(sObjectName, 7)) = "LDAP://" Then
		' is already an ADsPath
		sNameType = "adspath"

	Else
		' must be a name
		sNameType = "name"

	End If

	ADObject_NameType = sNameType
End Function

Function ADObject_Validate(ByVal sObjectADsPath)
	' Version 071122
	' Returns ADsPath of object as a way of validating that the object exists
	'
	' INPUTS:	sObjectADsPath		ADsPath of object to test
	' RETURNS:	ADObject_Validate	Path of object (if it exists) or blank
	
	Dim oObject
	On Error Resume Next
	Set oObject = GetObject(sObjectADsPath)
	If Err.Number <> 0 Then
		ADObject_Validate = ""
		Err.Clear
	Else
		ADObject_Validate = oObject.ADsPath
	End If	
End Function

Function AD_Search_Dictionary(sLDAPQuery)
	' Version 071204
	' Returns a dictionary of AD objects
	' A wrapper around AD_Search_RS
	' Inputs:	sLDAPQuery				The query to perform
	' Returns	AD_Search_Dictionary	Dictionary with KEY set to the first
	'									attribute returned by the search, and
	'									the item set to the second.
	'									The sLDAPQuery should specify a unique
	'									attribute (e.g. DN or ADsPath) for the
	'									first attribute!
	
	Dim rs, dResults, sKey, sItem
	Set dResults = CreateObject("Scripting.Dictionary")
	dResults.CompareMode = vbTextCompare ' Case INsensitive

	Set rs = AD_Search_RS(sLDAPQuery)
	If Not(rs Is Nothing) Then
		If Not (rs.BOF And rs.EOF) Then
			rs.MoveFirst
			Do While Not rs.EOF
				sKey = rs.Fields(0)
				sItem = ""
				On Error Resume Next ' Trap in case there is no second field
				sItem = rs.Fields(1)
				On Error GoTo 0
				dResults.Add sKey, sItem
				rs.MoveNext
			Loop
		End If
	Else
		' Handle error: Query returned no results
	End If
	Set AD_Search_Dictionary = dResults
	
End Function

Function AD_Search_Array(sLDAPQuery)
	' Version 071130
	' Returns an array of AD objects
	' A wrapper around AD_Search_RS
	' Inputs:	sLDAPQuery		The query to perform
	' Returns	AD_Search_Array	Array of Field(0) of recordset
	'							or empty array if recordset is empty
	
	Dim rs, aResults
	aResults = array()
	Set rs = AD_Search_RS(sLDAPQuery)
	If Not(rs Is Nothing) Then
		If Not (rs.BOF And rs.EOF) Then
			rs.MoveFirst
			Do While Not rs.EOF
				Redim Preserve aResults(ubound(aResults) + 1)
				aResults(ubound(aResults)) = rs.Fields(0)
				rs.MoveNext
			Loop
		End If
	Else
		' Error handling: query returned Nothing
	End If
	AD_Search_Array = aResults
	
End Function

Function AD_Search_RS(sLDAPQuery)
	' Version 071130
	' Returns a recordset of AD objects
	' Inputs:	sLDAPQuery		The query to perform
	' Returns:	AD_Search_RS	Recordset

	Dim oConnection
	' Open an ADO connection using null credentials
	Set oConnection = CreateObject("ADODB.Connection")
	oConnection.Provider = "ADsDSOObject"
	oConnection.Open "", vbNullString, vbNullString
	If oConnection.State = 0 Then ' 0 = adStateClosed
	     ' Error handling code: can't connect to AD
		Set AD_Search_RS = Nothing
	Else
		Set AD_Search_RS = oConnection.Execute(sLDAPQuery)
	End If
End Function

' #endregion