'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Proxy.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 11/30/2007
'
' USAGE:  
' cscript.exe Proxy.vbs
'
' Processes requests for the Proxy Framework
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
'==========================================================================

Option Explicit

' #region CONFIGURATION BLOCK
'==========================================================================
' DATABASE DEFINITION
' Define where our script will find its data store for the request queue
Dim sFile, sTable
sFile = "Provisioning.mdb"
sTable = "RequestQueue" ' Name of Excel sheet or Access table
Dim sStatusField, sCommandField, sParametersField, sRequestDateField
Dim sRequestByField, sExecuteDateField, sRetriesField, sNotesField
Dim sRequestFileField
' Data table (e.g. Excel worksheet) consists of fields/columns 
' labeled as defined below.  
' The items on the right side of the equals sign
' should be the labels in the first row of the worksheet
sStatusField = "Status"
sCommandField = "Command"
sParametersField = "Parameters"
sRequestDateField = "RequestDate"
sRequestByField = "RequestBy"
sExecuteDateField = "ExecuteDate"
sRetriesField = "Retries"
sNotesField = "Notes"
sRequestFileField = "RequestFile"
'--------------------------------------------------------------------------
' COMMAND EXECUTION
' When a script is run, how long do we wait before terminating it,
' and how many retries do we attempt (to overcome network connectivtity
' and other problems)
Dim iTimeout, iMaxRetries
iTimeout = 5
iMaxRetries = 3
'==========================================================================
' #endregion

' #region STANDARD STUFF
' Data ADO enums
Const adStateClosed = 0
Const adOpenStatic = 3
Const adOpenDynamic = 2
Const adLockOptimistic = 3
Const adUseClient = 3

Dim FSO : Set FSO = CreateObject("Scripting.FileSystemObject")
Dim WSHShell : Set WSHShell = CreateObject("WScript.Shell")
Dim sScriptPath : sScriptPath = ScriptPath()
Dim sAppPath :sAppPath = CreateObject("Scripting.FileSystemObject").GetParentFolderName(sScriptPath)

' DERIVE DOMAIN INFORMATION
Dim adSys, sComputerDN, sDomainDNS, sDomainNetBIOS, sDomainDN
Set adSys = CreateObject("ADSystemInfo")
sComputerDN = adSys.ComputerName
sDomainDN = Mid(sComputerDN, InStr(lcase(sComputerDN),",dc=") +1 )
sDomainDNS = adSys.DomainDNSName
sDomainNetBIOS = adSys.DomainShortName

' #end region

' Identify database type
Dim sFileType
Dim MSOfficeVersion
Call ADO_IdentifyDataType (sFile, sFileType, MSOfficeVersion)

' #region PROCESS INBOUND REQUESTS

' Connect to the database to add records based on recent requests
Dim sSQL, rsQueue
sFile = sScriptPath & "\" & sFile
sSQL = "SELECT * FROM " & sTable & " WHERE ID=0"
Set rsQueue = Queue_Query(sSQL)

' See if there are any new files in the folder
Dim oFolder, oFile, sRequestFile, sRequestFileName, dDateCreated
Dim sCommandLine, sParmsLine

Set oFolder = FSO.GetFolder(sAppPath & "\Requests")
For Each oFile In oFolder.Files
	sRequestFile = oFile.Path
	sRequestFileName = oFile.Name
	If ValidFile(oFile, sCommandLine, sParmsLine) Then
		dDateCreated = oFile.DateCreated
		rsQueue.AddNew
		rsQueue.Fields(sStatusField) = "open"
		rsQueue.Fields(sCommandField) = sCommandLine
		rsQueue.Fields(sParametersField) = sParmsLine
		rsQueue.Fields(sRequestDateField) = dDateCreated
		rsQueue.Fields(sRequestByField) = FileOwner(oFile.Path)
		rsQueue.Fields(sRequestFileField) = sRequestFileName
		rsQueue.Update
		oFile.Move sAppPath & "\Archives\" & sRequestFileName
	Else
		oFile.Move sAppPath & "\Errors\" & sRequestFileName
	End If
Next

' #end region

' #region EXECUTE OPEN TASK REQUESTS

' Look at all open tasks
Dim sScript, sParameters, sCommand, sStatus, sNotes, aResults, iRetries, sNotesCurrent
sSQL = "SELECT * FROM " & sTable & " WHERE Status=""open"""
Set rsQueue = Queue_Query(sSQL)
If Not (rsQueue.BOF And rsQueue.EOF) Then
	Do While Not rsQueue.EOF
		sCommand = rsQueue.Fields(sCommandField)
		sScript = GetScript(sCommand)

		iRetries = rsQueue.Fields(sRetriesField)
		If IsNull(iRetries) Then iRetries = 0

		If sScript = "" Then
			' COULD NOT FIND SCRIPT
			sStatus = "error"
			sNotes = "Could not find script for command: " & sCommand
			
		Else
			' RUN COMMAND
			sParameters = rsQueue.Fields(sParametersField)
			sParameters = sParameters & " /requestedBy:""" & rsQueue.Fields(sRequestByField) & """"
			sCommand = "cscript.exe //nologo """ & sScript & """ " & sParameters
			aResults = Execute_Capture(sCommand, iTimeout, True)
			
			If aResults(0) = 0 Then
				' Success
				sStatus = "complete"
				sNotes = aResults(1)			
			
			ElseIf aResults(0) < 1 Then
				' Timeout
				iRetries = iRetries + 1
				If iRetries >= iMaxRetries Then
					sStatus = "timeout"
				Else
					sStatus = "open"
				End If
				sNotes = aResults(1) & VbCrLf & aResults(2)
			
			Else
				' Error
				iRetries = iRetries + 1
				If iRetries >= iMaxRetries Then
					sStatus = "error"
				Else
					sStatus = "open"
				End If
				sNotes = sCommand & VbCrLf & aResults(1) & VbCrLf & aResults(2)
			End If
		End If
		
		' Log the results
		rsQueue.Fields(sStatusField) = sStatus
		rsQueue.Fields(sExecuteDateField) = Now()
		rsQueue.Fields(sRetriesField) = iRetries

		sNotesCurrent = rsQueue.Fields(sNotesField)
		If IsNull(sNotesCurrent) Then sNotesCurrent = ""
		sNotesCurrent = Now() & VbCrLf & sStatus & VbCrLf & sNotes & VbCrLf & VbCrLf & sNotesCurrent
		rsQueue.Fields(sNotesField) = sNotesCurrent

		rsQueue.Update
		rsQueue.MoveNext
	Loop
End If

' #end region

WScript.Quit (0)

' ==========================================================================
' COMMAND ROUTINES

Function GetScript(ByVal sCommand)
	Dim oScript
	GetScript = ""
	On Error Resume Next
	Set oScript = FSO.GetFile(sAppPath & "\Scripts\" & sCommand & ".vbs")
	GetScript = oScript.Path
	On Error GoTo 0
End Function

Function Execute_Capture(ByVal sCommand, ByVal iTimeout, ByVal bTerminate)
	' VERSION 071122
	' Executes command in sCommand
	' Times out after iTimeout and (optionally) terminates the process
	' Returns exit code, StdOut & StdErr
	'
	' USAGE:	arrayvariable = Execute_Capture(sCommand, iTimeout, [true|false])
	'
	' INPUTS:	sCommand: 			Command to run
	'			iTimeout:			Time (in seconds) to wait for process
	'			bTerminate:			TRUE  = terminate process after timeout
	'								FALSE = return from function
	' RETURNS:	Execute_Capture:	*ARRAY*
	'				(0): 			Exit code of command 
	'								0.01 = timed out but NOT terminated
	'								0.99 = timed out and TERMINATED
	'				(1):			StdOut
	'				(2):			StdErr
	'
	' REQUIRES:	global definition of WSHShell
	'			or
	'			uncomment next line:
	' 			Dim WSHShell : Set WSHShell = CreateObject("WScript.Shell")
	
	Dim oExec, iTimer, iExitCode, sOut, sErr
	
	On Error Resume Next
	Set oExec = WSHShell.Exec(sCommand)
	If Err.Number <> 0 Then 
		' There was a problem running the command itself
		iExitCode = Err.Number
		sStdOut = ""
		sStdErr = "Failed to run command " & sCommand & VbCrLf & Err.Description
		Execute_Capture = Array(iExitCode, sOut, sErr)
		Exit Function
	End if
	
	' Command was launched. Wait for completion or timeout.
	On Error Goto 0
	iTimer = 0
	Do While oExec.Status = 0
		WScript.Sleep 100
		iTimer = iTimer + 100
		If (iTimer/1000) > iTimeout Then Exit Do
	Loop
	
	If oExec.Status = 0 Then
		' Not completed, but timed out
		If bTerminate Then
			' Terminate
			oExec.Terminate
			iExitCode = 0.99
		Else
			' Timeout but do not terminate
			iExitCode = 0.01
		End If
	Else
		' Completed
		iExitCode = oExec.ExitCode
	End If

	sOut = oExec.StdOut.ReadAll
	sErr = oExec.StdErr.ReadAll
	Execute_Capture = Array(iExitCode, sOut, sErr)
	
End Function

' ==========================================================================
' DATABASE ROUTINES
' The following routines are related to the database functionality

Function Queue_Query(sSQL)
	' Connects to the Queue defined in the CONFIGURATION BLOCK above
	Dim sConnectionString
	sConnectionString = ADO_BuildConnectionString(sFileType,sFile,"","","")
	Dim oConnection
	Set oConnection = ADO_DBConnection(sConnectionString)
	Dim rs
	'On Error Resume Next
	Set rs = ADO_GetRecordset(sSQL, oConnection)
	If Err.Number <> vbEmpty Then
		sScriptResults = sScriptResults & ErrorHandler("Queue_Query", "Could not open Queue " & sFile & " with SQL " & sSQL)
		Set rs=Nothing
		Err.Clear
	Else
		Set Queue_Query = rs
	End If
	On Error GoTo 0
End Function

Sub ADO_IdentifyDataType (sFile, sFileType, MSOfficeVersion)
	' Takes an INPUT of sFile (path/filename of an Excel or Access file)
	' and returns  the type ("excel" or "access") and the version (2003 or 2007)
	' to the sFileType and MSOfficeVersion variables
	
	' Determine file type (crude: there are slicker ways)
	On Error Resume Next
	sFileType = Mid(sFile, InStrRev(sFile,".") +1)
	If Err.Number<>0 Then
		' Error Handling
		WScript.Quit (Err.Number) ' Throw an error
	End If
	On Error GoTo 0
	
	sFileType = LCase(sFileType)
	Select Case sFileType
		Case "xls"
			MSOfficeVersion = 2003
			sTable = "[" & sTable & "$]" ' Must format sheet name to make data table
			sFileType = "excel"
		Case "xlsx"
			MSOfficeVersion = 2007
			sTable = "[" & sTable & "$]" ' Must format sheet name to make data table
			sFileType = "excel"
		Case "mdb"
			MSOfficeVersion = 2003
			sFileType = "access"
		Case "accdb"
			MSOfficeVersion = 2007
			sFileType = "access"
		Case Else
			' Error Handling
			WScript.Quit (501) ' Throw an error	(I choose 501 as my standard error number)	
	End Select
End Sub

Function ADO_BuildConnectionString(ByVal sType, ByVal sFile, ByVal sDatabase, ByVal sUsername, ByVal sPassword)
	' Returns a connection string for Access, Excel or SQL
	' Example usage:
	' 	ACCESS: sConnectionString = ADO_BuildConnectionString("access",filename,"","","")
	'	EXCEL:	sConnectionString = ADO_BuildConnectionString("excel",filename,"","","")
	Dim sProvider, sExtendedProperties
	Select Case MSOfficeVersion
		Case 2007
			sProvider = "Provider=Microsoft.ACE.OLEDB.12.0;"
			sExtendedProperties = "Extended Properties=Excel 12.0;"
		Case Else	
			sProvider = "Provider=Microsoft.Jet.OLEDB.4.0;" ' FOR PREVIOUS VERSIONS
			sExtendedProperties = "Extended Properties=Excel 8.0;"
	End Select
    Select Case lcase(sType)
        Case "access"
            ADO_BuildConnectionString = sProvider & "Data Source=" & sFile & ";"
        Case "excel"
            ADO_BuildConnectionString = sProvider & "Data Source=" & sFile & ";" & sExtendedProperties
        Case "sql"
            ADO_BuildConnectionString = "Provider=sqloledb;server=" & sFile & ";database=" & sDatabase & ";uid=" & sUsername & ";pwd=" & sPassword & ";"
    End Select   
End Function

Function ADO_DBConnection(ByVal sConnectionString)
	' Returns an ADODB.Connection object
	' Example usage: 
	'	Set oConnection = ADO_DBConnection(sConnectionString)
	Dim oConnection ' As ADODB.Connection
    Set oConnection = CreateObject("ADODB.Connection")
    oConnection.Open sConnectionString
    Set ADO_DBConnection = oConnection
End Function

Function ADO_GetRecordset(ByVal strSQL, ByRef oDBConnection)
	' Returns an ADODB.Recordset object
	' Example usage:
	'	Set rsDel = ADO_GetRecordset(sSQL, oConnection)
	    Dim rs
	    Set rs = CreateObject("adodb.Recordset")
	    rs.CursorLocation = adUseClient
		rs.Open strSQL, oDBConnection, adOpenStatic, adLockOptimistic
	    Set ADO_GetRecordset = rs
End Function

' ==========================================================================
Function ScriptPath()
	' This works over network and local paths
	ScriptPath = CreateObject("Scripting.FileSystemObject").GetParentFolderName(Wscript.ScriptFullName)
End Function

Function FileOwner(ByVal sFileSpec)
	Dim oWMI, colItems, oItem, sComputer
	sComputer = "."
	
	set oWMI = GetObject("winmgmts:\\" & sComputer & "\root\cimv2")
	set colItems = oWMI.ExecQuery _
	    ("ASSOCIATORS OF {Win32_LogicalFileSecuritySetting='" & sFileSpec & "'}" _ 
	        & " WHERE AssocClass=Win32_LogicalFileOwner ResultRole=Owner")
	for each oItem in colItems
	    FileOwner = oItem.ReferencedDomainName & "\" & oItem.AccountName
	Next
End Function

Function ValidFile(ByRef oFile, ByRef sCommandLine, ByRef sParmsLine)
	' Determine whether the request file looks valid.
	' CRITERIA: Line 1 is command, and matches a script in the Scripts folder
	'			Line 2 is the parameter line (can be blank, but must exist)
	
	Dim bValid, oTextFile, sTextLine1, oScript, sTextLine2
	
	On Error Resume Next
	set oTextFile = oFile.OpenAsTextStream
	sTextLine1 = oTextFile.ReadLine
	Set oScript = FSO.GetFile(sAppPath & "\Scripts\" & sTextLine1 & ".vbs")
	sTextLine2 = oTextFile.ReadLine
	
	If Err.Number = 0 Then
		sCommandLine = sTextLine1
		sParmsLine = sTextLine2
		bValid = True
	Else
		sCommandLine = ""
		sParmsLine = ""
		bValid = False
		Err.Clear
	End If
	On Error GoTo 0
	
	ValidFile = bValid
	
End Function
