'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Software_Deploy.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 11/23/2007
'
' USAGE:  
' cscript.exe Software_Deploy.vbs
'
' COMMENT: Used to deploy Office 2007
'		   Or to perform any other command on systems
'		   Be sure to change elements in the CONFIGURATION BLOCK below
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
' You are free to use this script and make derivatives of it,
' but you may not remove any of this header.
' We also ask that any functional improvements be reported to
' Intelliem at the URL above for the benefit of all.
'==========================================================================

Option Explicit

'==========================================================================
' BEGIN CONFIGURATION BLOCK

' DOMAIN
Dim sDomainDN
sDomainDN = "dc=contoso,dc=com"

' DATABASE DEFINITION
' Define where our script will find its data store for logging
' The database is used to log the results of the script
' It can be an Access databse or an Excel worksheet
' Because systems don't yet have Office 2007, 
' use a downlevel database format (xls/mdb)
Dim sFile, sTable
sFile = "\\contoso.com\software\microsoft\office 2007\" & _
	"Contoso_Setup\SystemConfigurationDB.mdb"
sTable = "Log" ' Name of Access table or Excel sheet within the file
Dim sComputerNameField, sActionField, sStatusField, sDateField, sNotesField
' Data table (e.g. Access table/Excel worksheet) consists of fields/columns 
' labeled as defined below.  
' The items on the right side of the equals sign
' should be the labels in the first row of the worksheet
sComputerNameField = "ComputerName"
sActionField = "Action"
sStatusField = "Status"
sDateField = "Date"
sNotesField = "Notes"

Dim sCommand
' COMMAND to run Office 2007 installation
' string starts and ends with quotes
' wherever you need quotes IN the command, use two sets of quotes
sCommand = """\\contoso.com\software\microsoft\office 2007\setup.exe"" " & _
	"/adminfile ""\\contoso.com\software\microsoft\office 2007\" & _
	"Contoso_Setup\Contoso_Standard_Configuration.msp"""
Dim iTimeout
' How long to wait for Office installation to complete (in SECONDS)
' or zero to wait indefinitely
iTimeout = 20 * 60	' 20 minutes

Dim sAction
' ACTION that will be logged in the ACTION column of the log
sAction = "Office 2007 Deployment"

Dim sStagingGroup, sSuccessGroup, sErrorGroup
' GROUPS that manage this change
' Important: These groups must have the access control entry
'     SELF::Allow::Write::Members
' Group that computer/user is in BEFORE this script is run
sStagingGroup = "CCM_Office 2007 Deploy" 
' Group that this computer/user is moved to based on success
sSuccessGroup = "APP_Office 2007"
sErrorGroup = "ALERT_Office 2007 Deploy"

Dim bDebugMode, sDebugFile
' DEBUG MODE: TRUE (on) FALSE (off)
' if on, run the script then look for "C:\Software_Deploy_Log.txt"
bDebugMode = True
' Filename to dump results to.  Set to ECHO to dump results to console.
sDebugFile = "C:\Software_Deploy_Log.txt"

' END CONFIGURATION BLOCK
'==========================================================================

' Data ADO enums
Const adStateClosed = 0
Const adOpenStatic = 3
Const adOpenDynamic = 2
Const adLockOptimistic = 3
Const adUseClient = 3
Dim retVal

' Perform logic to translate configuration to data table identity
Dim sFileType
Dim MSOfficeVersion
Call ADO_IdentifyDataType (sFile, sFileType, MSOfficeVersion)

' Initialization
Dim sComputerName
sComputerName = GetComputerName()
Dim WSHShell
Set WSHShell = CreateObject("WScript.Shell")
Dim FSO
Set FSO = CreateObject("Scripting.FileSystemObject")
Dim sScriptResults

' Run the command and terminate the installation
' if it does not complete by the timeout
Dim aResults, sStdErr, sStdOut, iExitCode
aResults = Execute_Capture(sCommand, iTimeout, True)
iExitCode = aResults(0)
sStdOut   = aResults(1)
sStdErr   = aResults(2)

' Interpret the results
Dim sStatus, sNotes
Select Case iExitCode
	Case 0
		sStatus = "SUCCESS"
		sNotes = ""
		sScriptResults = sScriptResults & "SUCCESS" & VbCrLf 
	Case Else
		sStatus = "ERROR"
		sNotes  = iExitCode & ": " & sStdErr
		sScriptResults = sScriptResults & "ERROR" & VbCrLf & sNotes & VbCrLf
End Select

' Log the results
Call Log_WriteCommandResults(sComputerName, sStatus, sNotes)

' Modify groups to reflect results
Select Case sStatus
	Case "SUCCESS"
		retVal= Group_AddMember (sSuccessGroup, sComputerName)
	Case "ERROR"
		retVal = Group_AddMember (sErrorGroup, sComputerName)
End Select
retVal = Group_RemoveMember (sStagingGroup, sComputerName)

' -------------------------------------------------------------------------
' Wrap up the script

' DEBUG
If bDebugMode Then
	If ucase(sDebugFile) = "ECHO" Then
		WScript.Echo sScriptResults
	Else
		retVal = WriteFile(sDebugFIle, sScriptResults, True)
	End If
End If

WScript.Quit(0)

Function Execute_Capture(ByVal sCommand, ByVal iTimeout, ByVal bTerminate)
	' VERSION 071122
	' Executes command in sCommand
	' Times out after iTimeout and (optionally) terminates the process
	' Returns exit code, StdOut & StdErr
	'
	' USAGE:	arrayvariable = Execute_Capture(sCommand, iTimeout, [true|false])
	'
	' INPUTS:	sCommand: 			Command to run
	'			iTimeout:			Time (in seconds) to wait for process
	'			bTerminate:			TRUE  = terminate process after timeout
	'								FALSE = return from function
	' RETURNS:	Execute_Capture:	*ARRAY*
	'				(0): 			Exit code of command 
	'								0.01 = timed out but NOT terminated
	'								0.99 = timed out and TERMINATED
	'				(1):			StdOut
	'				(2):			StdErr
	'
	' REQUIRES:	global definition of WSHShell
	'			or
	'			uncomment next line:
	' 			Dim WSHShell : Set WSHShell = CreateObject("WScript.Shell")
	
	Dim oExec, iTimer, iExitCode, sOut, sErr
	
	On Error Resume Next
	sScriptResults = sScriptResults & sCommand & VbCrLf 
	Set oExec = WSHShell.Exec(sCommand)
	If Err.Number <> 0 Then 
		' There was a problem running the command itself
		iExitCode = Err.Number
		sStdOut = ""
		sStdErr = "Failed to run command " & sCommand & VbCrLf & Err.Description
		Execute_Capture = Array(iExitCode, sStdOut, sStdErr)
		Err.Clear
		Exit Function
	End if
	
	' Command was launched. Wait for completion or timeout.
	On Error Goto 0
	iTimer = 0
	Do While oExec.Status = 0
		WScript.Sleep 100
		iTimer = iTimer + 100
		If (iTimer/1000) > iTimeout Then Exit Do
	Loop
	
	If oExec.Status = 0 Then
		' Not completed, but timed out
		If bTerminate Then
			' Terminate
			oExec.Terminate
			iExitCode = 0.99
		Else
			' Timeout but do not terminate
			iExitCode = 0.01
		End If
	Else
		' Completed
		iExitCode = oExec.ExitCode
	End If

	sOut = oExec.StdOut.ReadAll
	sErr = oExec.StdErr.ReadAll
	Execute_Capture = Array(iExitCode, sOut, sErr)
	
End Function

' ==========================================================================
' LOG ROUTINES
' The following routines are related to the LOGGING functionality

Sub Log_WriteCommandResults (sComputerName, sStatus, sNotes)
	' Write entry to Log
	' First, connect to log. By using the sql query below, we get an empty recordset
	' which is better for performance since we're only appending a record, anyway
	Dim rsLog, sSQL
	sSQL = "SELECT * FROM " & sTable & " WHERE " & sComputerNameField & "=''"
	Set rsLog = Log_Query(sSQL)
	If Not (rsLog Is Nothing) Then
		rsLog.AddNew
		rsLog.Fields(sComputerNameField) = sComputerName
		rsLog.Fields(sActionField) = sAction
		rsLog.Fields(sDateField) = Now()
		rsLog.Fields(sStatusField) = sStatus
		rsLog.Fields(sNotesField) = sNotes
		On Error Resume Next
		rsLog.Update
		If Err.Number <> 0 Then
			sScriptResults = sScriptResults & _
				ErrorHandler("Log_WriteCommandResults", "Could not write to log.")
			Exit Sub
		End If
		rsLog.Close
		Set rsLog = Nothing
	Else
		sScriptResults = sScriptResults & "ERROR: Error querying log" & VbCrLf & sSQL & VbCrLf
	End If
	
End Sub

Function Log_Query(sSQL)
	' Connects to the log defined in the CONFIGURATION BLOCK above
	Dim sConnectionString
	sConnectionString = ADO_BuildConnectionString(sFileType,sFile,"","","")
	Dim oConnection
	Set oConnection = ADO_DBConnection(sConnectionString)
	Dim rs
	On Error Resume Next
	Set rs = ADO_GetRecordset(sSQL, oConnection)
	If Err.Number <> vbEmpty Then
		sScriptResults = sScriptResults & ErrorHandler("Log_Query", "Could not open log " & sFile & " with SQL " & sSQL)
		Set rs=Nothing
		Err.Clear
	Else
		Set Log_Query = rs
	End If
	On Error GoTo 0
End Function

Sub ADO_IdentifyDataType (sFile, sFileType, MSOfficeVersion)
	' Takes an INPUT of sFile (path/filename of an Excel or Access file)
	' and returns  the type ("excel" or "access") and the version (2003 or 2007)
	' to the sFileType and MSOfficeVersion variables
	
	' Determine file type (crude: there are slicker ways)
	On Error Resume Next
	sFileType = Mid(sFile, InStrRev(sFile,".") +1)
	If Err.Number<>0 Then
		' Error Handling
		WScript.Quit (Err.Number) ' Throw an error
	End If
	On Error GoTo 0
	
	sFileType = LCase(sFileType)
	Select Case sFileType
		Case "xls"
			MSOfficeVersion = 2003
			sTable = "[" & sTable & "$]" ' Must format sheet name to make data table
			sFileType = "excel"
		Case "xlsx"
			MSOfficeVersion = 2007
			sTable = "[" & sTable & "$]" ' Must format sheet name to make data table
			sFileType = "excel"
		Case "mdb"
			MSOfficeVersion = 2003
			sFileType = "access"
		Case "accdb"
			MSOfficeVersion = 2007
			sFileType = "access"
		Case Else
			' Error Handling
			WScript.Quit (501) ' Throw an error	(I choose 501 as my standard error number)	
	End Select
End Sub

Function ADO_BuildConnectionString(ByVal sType, ByVal sFile, ByVal sDatabase, ByVal sUsername, ByVal sPassword)
	' Returns a connection string for Access, Excel or SQL
	' Example usage:
	' 	ACCESS: sConnectionString = ADO_BuildConnectionString("access",filename,"","","")
	'	EXCEL:	sConnectionString = ADO_BuildConnectionString("excel",filename,"","","")
	Dim sProvider, sExtendedProperties
	Select Case MSOfficeVersion
		Case 2007
			sProvider = "Provider=Microsoft.ACE.OLEDB.12.0;"
			sExtendedProperties = "Extended Properties=Excel 12.0;"
		Case Else	
			sProvider = "Provider=Microsoft.Jet.OLEDB.4.0;" ' FOR PREVIOUS VERSIONS
			sExtendedProperties = "Extended Properties=Excel 8.0;"
	End Select
    Select Case lcase(sType)
        Case "access"
            ADO_BuildConnectionString = sProvider & "Data Source=" & sFile & ";"
        Case "excel"
            ADO_BuildConnectionString = sProvider & "Data Source=" & sFile & ";" & sExtendedProperties
        Case "sql"
            ADO_BuildConnectionString = "Provider=sqloledb;server=" & sFile & ";database=" & sDatabase & ";uid=" & sUsername & ";pwd=" & sPassword & ";"
    End Select   
End Function

Function ADO_DBConnection(ByVal sConnectionString)
	' Returns an ADODB.Connection object
	' Example usage: 
	'	Set oConnection = ADO_DBConnection(sConnectionString)
	Dim oConnection ' As ADODB.Connection
    Set oConnection = CreateObject("ADODB.Connection")
    oConnection.Open sConnectionString
    Set ADO_DBConnection = oConnection
End Function

Function ADO_GetRecordset(ByVal strSQL, ByRef oDBConnection)
	' Returns an ADODB.Recordset object
	' Example usage:
	'	Set rsDel = ADO_GetRecordset(sSQL, oConnection)
	    Dim rs
	    Set rs = CreateObject("adodb.Recordset")
	    rs.CursorLocation = adUseClient
		rs.Open strSQL, oDBConnection, adOpenStatic, adLockOptimistic
	    Set ADO_GetRecordset = rs
End Function

' ==========================================================================
' GROUP MEMBERSHIP ROUTINES
' The following routines manipulate group membership

Function Group_AddMember (ByVal sGroupName, ByVal sMemberName)
	' Adds a member to a group
	' Inputs:	sGroupName	 	samAccountName of the group
	'			sMemberName 	samAccountName (user/group) or Name (computer) of member to add
	' Requires:	sDomainDN		the DN of the domain
	'			sScriptResults	global declaration - accumulates script results
	' 			ErrorHandler	function
	
	Dim sGroupADsPath, oGroup, sMemberADsPath, oMember
	sGroupADsPath = ADObject_Find_UCG(sGroupName, sDomainDN)
	If sGroupADsPath = "" Then
		Group_AddMember = False
		sScriptResults = sScriptResults & ErrorHandler("Group_AddMember", "Could not find group: " & sGroupName)
		Exit Function
	End If
	
	On Error Resume Next
	Set oGroup = GetObject(sGroupADsPath)
	If Err.Number <> vbEmpty Then
		Group_AddMember = False
		sScriptResults = sScriptResults & ErrorHandler ("Group_AddMember", "Could not connect to group: " & sGroupName)
		Err.Clear
		Exit Function
	End If
	
	sMemberADsPath = ADObject_Find_UCG(sMemberName, sDomainDN)
	If sMemberADsPath = "" Then
		Group_AddMember = False
		sScriptResults = sScriptResults & ErrorHandler ("Group_AddMember", "Could not find: " & sMemberName)
		Exit Function
	End If
	
	oGroup.Add(sMemberADsPath)
	If Err.number <> vbEmpty Then
		Group_AddMember = False
		sScriptResults = sScriptResults & _
			ErrorHandler ("Group_AddMember", "Could not add: " & sMemberName & " (" & sMemberADsPath & ") " & _
			" to group " & sGroupName & " (" & sGroupADsPath & ")")
		Exit Function
	End If
	Group_AddMember = True
	
End Function

Function Group_RemoveMember (ByVal sGroupName, ByVal sMemberName)
	' Removes a member from a group
	' Inputs:	sGroupName		samAccountName of the group
	' 			sMemberName 	samAccountName (user/group) or Name (computer) of member to add
	' Requires:	sDomainDN 		the DN of the domain
	'			sScriptResults	global declaration - accumulates script results
	' 			ErrorHandler 	function
	
	Dim sGroupADsPath, oGroup, sMemberADsPath, oMember
	sGroupADsPath = ADObject_Find_UCG(sGroupName, sDomainDN)
	If sGroupADsPath = "" Then
		Group_RemoveMember = False
		sScriptResults = sScriptResults & ErrorHandler("Group_RemoveMember", "Could not find group: " & sGroupName)
		Exit Function
	End If
	
	On Error Resume Next
	Set oGroup = GetObject(sGroupADsPath)
	If Err.Number <> vbEmpty Then
		Group_RemoveMember = False
		sScriptResults = sScriptResults & ErrorHandler ("Group_RemoveMember", "Could not connect to group: " & sGroupName)
		Err.Clear
		Exit Function
	End If
	
	sMemberADsPath = ADObject_Find_UCG(sMemberName, sDomainDN)
	If sMemberADsPath = "" Then
		Group_RemoveMember = False
		sScriptResults = sScriptResults & ErrorHandler ("Group_RemoveMember", "Could not find: " & sMemberName)
		Exit Function
	End If
	
	oGroup.Remove(sMemberADsPath)
	If Err.number <> vbEmpty Then
		Group_RemoveMember = False
		sScriptResults = sScriptResults & _
			ErrorHandler ("Group_RemoveMember", "Could not remove " & sMemberName & " (" & sMemberADsPath & ") " & _
			" from group " & sGroupName & " (" & sGroupADsPath & ")")
		Exit Function
	End If
	Group_RemoveMember = True 
	
End Function

Function ADObject_Find_UCG(sObjectName, sSearchDN)
	' VERSION 070706
	' Returns the full ADsPath (LDAP://...) of a user, computer, or group
	' Inputs:
	' 		sObjectName: The unique identifier for the object class.  The script supports:
	'			User: 		sAMAccountName (pre-Windows 2000 logon name)
	'			Group: 		sAMAccountName (pre-Windows 2000 logon name)
	'			Computer: 	Name (translated by the script to the computer's sAMAccountName by adding a $)
	' 		sSearchDN: the DN within which to search (often, the DN of the domain, e.g. dc=contoso, dc=com)

	Dim oConnection
	Dim oRecordset
	Dim sLDAPObjectQuery
	Dim sLDAPIdentifierQuery
	Dim sLDAPQuery
	Dim sProperties
	Dim oADObject
	Dim aProperties
	Dim sProperty
	Dim sLDAPIdentifier
	Dim sSearchScope
	
	sLDAPIdentifier = "samAccountName"
	sProperties = "ADsPath"
	sSearchScope = "subtree"
	
	' Open an ADO connection using null credentials
	Set oConnection = CreateObject("ADODB.Connection")
	oConnection.Provider = "ADsDSOObject"
	'On Error Resume Next
	oConnection.Open "", vbNullString, vbNullString
	If oConnection.State = 0 Then ' 0 = adStateClosed
	     ' Error handling code: can't connect to AD
	     ADObject_Find_UCG = ""
	     Exit Function
	End If
	
	' Build the LDAP Query
	sLDAPQuery = "<LDAP://" & sSearchDN & ">;"
	sLDAPQuery = sLDAPQuery & _
		"(|(samAccountName=" & sObjectName & ")" & _
		"(samAccountName=" & sObjectName & "$));" & _
		sProperties & ";" & sSearchScope
	
	' Retrieve the result set, close the connection, and check to make
	' sure we received at least one result
	Set oRecordset = oConnection.Execute (sLDAPQuery)
	If oRecordset.EOF and oRecordset.BOF Then
		' Error handling code: no object found
	    ADObject_Find_UCG = ""
	     Exit Function
	End If
	
	ADObject_Find_UCG = oRecordset.Fields("ADsPath")
	oRecordset.Close
	oConnection.Close
	
End Function

' ==========================================================================
' MISCELLANEOUS

Function ErrorHandler( ByVal sRoutine, ByVal sErrorText )
	' Returns a string describing the error and clears Err
    Dim sErrorHandler
    sErrorHandler = String( 50, "-") & VbCrLf & _
    	"Error in: " & sRoutine & VbCrLf & _
    	sErrorText & VbCrLf & _
		"Error number: " & Err.number & VbCrLf & _
    	"Error Description: " & Err.Description & vbCrLf
    Err.Clear
    ErrorHandler = sErrorHandler  
End Function

Function GetComputerName()
    Dim WshNetwork
    Set WshNetwork = WScript.CreateObject("WScript.Network")
    GetComputerName = WshNetwork.ComputerName 
End Function

Function WriteFile(ByVal FilePath, ByVal Text, ByVal Overwrite)
	WriteFile = False
	
	Const ForWriting = 2, ForAppending = 8, TristateUseDefault = -2
	Dim FSO, oFile, IOMode
	
	If Overwrite Then 
		IOMode = ForWriting
	Else
		IOMode = ForAppending
	End If
	
	Set FSO = CreateObject("Scripting.FileSystemObject")
	If FSO.FolderExists( FSO.GetParentFolderName(FilePath) ) Then
		On Error Resume Next
		Set oFile = FSO.OpenTextFile(FilePath, IOMode, _
			True, TristateUseDefault)
		oFile.Write Text
		oFile.Close
		If Err then
			Err.Clear
			WriteFile = False
		Else
			WriteFile = True
		End if
	End If
	
End Function