'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Software_Setup_Core.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 11/23/2007
'
' USAGE:  
' cscript.exe Software_Setup_Core.vbs
'
' COMMENT: Used to deploy an application
' This script is used to perform application installations.
' It is designed to be INCLUDED as common code in each
' Software_Setup.vbs
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
' You are free to use this script and make derivatives of it,
' but you may not remove any of this header.
' We also ask that any functional improvements be reported to
' Intelliem at the URL above for the benefit of all.
'==========================================================================

Dim iExitCode, sResults
Dim WSHShell, sCurrDir

Sub Initialize()
	Set WSHShell = CreateObject("WScript.Shell")
	sCurrDir = CreateObject("Scripting.FileSystemObject").GetParentFolderName(Wscript.ScriptFullName)
End Sub

Sub Replace_Paths()
	sWorkingDir = Replace(sWorkingDir, "%CD%", sCurrDir)
	sCommand = Replace(sCommand, "%WD%", sWorkingDir)
	sCommand = Replace(sCommand, "%CD%", sCurrDir)
	sCommandArgs = Replace(sCommandArgs, "%WD%", sWorkingDir)
	sCommandArgs = Replace(sCommandArgs, "%CD%", sCurrDir)
End Sub

Sub Install()
	' VERSION 070715
	' Performs an installation
	' To be "included" in all installation scripts
	
	Call Replace_Paths()
	
	Select Case UCASE(sCommandType)
		Case "MSI"
			if sCommandArgs = "" then sCommandArgs = "/qb- /norestart"
			sCommand = "msiexec.exe /i" & _
				" """ & sCommand & """ " & sCommandArgs
			iExitCode = Execute_Capture()

		Case "EXE"
			sCommand = """" & sCommand & """ " & sCommandArgs
			iExitCode = Execute_Capture()

		Case Else
			WScript.Echo "Command Type Unknown"
			WScript.Quit (500) ' Throw an exit code
	End Select

	If Right(sSuccessCodes,1) <> "," Then sSuccessCodes = sSuccessCodes & ","
	if instr(sSuccessCodes, iExitCode & ",") Then
		' Nothing - will exit sub and return to main code
	else
		WScript.Echo "ERROR " & iExitCode & VbCrLf & sResults
		WScript.Quit (iExitCode)
	end if
	
End Sub

Function Execute_Capture()
	' VERSION 070715
	' Executes command in sCommand
	' Returns combined output of StdOut & StdErr in sResults
	' and returns value of function = command exit code
	' INPUTS:	sCommand: command to run
	'			sResults: OUT parameter
	' RETURNS:	Execute_Capture: Exit code of command
	'			sResults: Combined output of StdOut & StdErr
	
	Dim oExec, iExitCode, sOutput
	
	On Error Resume Next
	
	' DEBUG CODE
	'MsgBox sCommand
	'Exit Function
	
	Set oExec = WSHShell.Exec(sCommand)
	
	' Error Trap
	If Err.Number<>0 Then
		sResults = Err.Description & VbCrLf & "COMMAND: " & sCommand
		Execute_Capture = Err.Number
		Exit Function
	End If
	On Error GoTo 0
	
	Do While oExec.Status = 0
		WScript.Sleep 100
	Loop
	
	iExitCode = oExec.ExitCode
	sOutput = oExec.StdOut.ReadAll
	
	If iExitCode <> 0 Then _
		sOutput = sOutput & VbCrLf & oExec.StdErr.ReadAll
		
	sResults = sOutput
	Execute_Capture = iExitCode
	
End Function

Function Find_Setup(ByVal sFileExtension)
	' Determines which file is the setup file by
	' looking for the first file it can find with an
	' extension that matches the specified extension type.
	' 
	' This is used only by very simple installers, to
	' save the effort of having to hard-code the
	' setup file name in the installer code itself.
	'
	' Obviously for more complicated software, looking
	' for the first "exe" or "msi" would not be advisable,
	' so hard-code the file name in the installer for those
	' applications, rather than call Find_Setup.
	
	Dim oFSO, oFolder, sFileName, sFileExt, oFile

	Call Replace_Paths()
	Set oFSO = CreateObject("Scripting.FileSystemObject")
	' MsgBox "Find_Setup Looking in: " & sWorkingDir
	Set oFolder = oFSO.GetFolder(sWorkingDir)
	
	For Each oFile In oFolder.Files
		sFileExt = oFSO.GetExtensionName(oFile.Path)
		' WScript.Echo oFile.Name & " - " & sFileExt
		If lcase(sFileExt) = lcase(sFileExtension) Then
			Find_Setup = oFile.Name
			Exit Function
		End If
	Next
	
End Function

Function Find_Folder()
	' Finds the "newest version" by looking for the folder with the greatest name
	' The routine grabs the names of all folders within the folder sCurrDir,
	' sorts them in descending order, then returns the first one it finds where
	' the first character is numeric (0-9), which would be the "highest" version
	
	' WARNING: Logic does not account for folders with names that begin with 
	' non-numeric characters, so folder names like v1.0 won't be returned correctly
	
	Dim oFSO, oFolder, oFolders, oSubFolder, aSubFolders, i, sFolder
	
	Set oFSO = CreateObject("Scripting.FileSystemObject")
	Set oFolder = oFSO.GetFolder(sCurrDir)
	
	If oFolder.SubFolders.Count = 0 Then
		Find_Folder = ""
	Else
		Set oFolders = CreateObject("Scripting.Dictionary")
		oFolders.CompareMode = vbTextCompare ' Case INsensitive
		For Each oSubFolder In oFolder.SubFolders
			oFolders.Add oSubFolder.Name, True
		Next
		
		aSubFolders = oFolders.Keys
		aSubFolders = Array_Sort(aSubFolders)
		For i = UBound (aSubFolders) To 0 Step -1
			sFolder = aSubFolders(i)
			If Left(sFolder,1) >= "0" And Left(sFolder,1)<="9" Then
				Find_Folder = sFolder
				Exit For
			End If
			Find_Folder = aSubFolders(UBound(aSubFolders))
		Next
	End If
	Find_Folder = sCurrDir & "\" & Find_Folder
	
End Function
	
Function Array_Sort(ByVal aArray) 
		' VERSION 070701
		' Sorts an array, very quickly, using .NET objects
		' INPUTS:	aArray: an array (incl Dictionary object) requiring sort
		' RETURNS:	Array_Sort: sorted
		' REQUIRES:	.NET Framework 2.0
		
	    Dim oNAL
	    Dim nIdx
	    
	    Set oNAL = CreateObject("System.Collections.ArrayList")
	    For nIdx = 0 To UBound(aArray)
	        oNAL.Add aArray(nIdx)
	    Next
	    
	    oNAL.Sort
	    
	    For nIdx = 0 To UBound(aArray)
	        aArray(nIdx) = oNAL(nIdx) 
	    Next 
	    
	    Array_Sort = aArray
	    
	End Function

Function Open_Serial(ByVal sFileName)
	' This function opens a file containing the serial number
	' for an application. The filename is passed as sFileName.
	' If <blank> is passed to the function, the function is
	' hard-coded to look for a file called Serial.txt
	'
	' I use this routine only in INTERACTIVE setups of 
	' one-off applications where I need to manually enter
	' a product key. The Serial.txt file opens and I can
	' either copy and paste it into the setup wizard or
	' refer to it as I type.
		
	Dim sDefaultFile
	sDefaultFile = "Serial.txt"
	
	Dim oFSO, oFolder, oFolders, oSubFolder, aSubFolders
	Dim sNotepad

	If sFileName = "" Then	
		Set oFSO = CreateObject("Scripting.FileSystemObject")
		Set oFolder = oFSO.GetFolder(sWorkingDir)
		If oFSO.FileExists(sWorkingDir & "\" & sDefaultFile) Then
			sFileName = sWorkingDir & "\" & sDefaultFile
		Else
			Exit Function
		End If
	End If

	sNotepad = "Notepad.exe """ & sFileName & """"
	WSHShell.Run sNotepad,1,True
End Function