'==========================================================================
'
' VBScript Source File -- Created with SAPIEN Technologies PrimalScript 2007
'
' NAME: Software_User_Staging.vbs
'
' AUTHOR: Dan Holme , Intelliem
' DATE  : 11/23/2007
'
' USAGE:  
' cscript.exe Software_User_Staging.vbs
'
' COMMENT: Sample that prompts the user to accept installation of
' a piece of software. When the user accepts, the computer is added
' to the appropriate group so that the sofware will be installed,
' and the user is removed from the group that is causing this
' script to be run as a logon script.
'
' See the Windows Administration Resource Kit for documentation
'
' Neither Microsoft nor Intelliem guarantee the performance
' of scripts, scripting examples or tools.
'
' See www.intelliem.com/resourcekit for updates to this script
'
' (c) 2007 Intelliem, Inc
' You are free to use this script and make derivatives of it,
' but you may not remove any of this header.
' We also ask that any functional improvements be reported to
' Intelliem at the URL above for the benefit of all.
'==========================================================================
Option Explicit

'==========================================================================
' BEGIN CONFIGURATION BLOCK

' DOMAIN
Dim sDomainDN
sDomainDN = "dc=contoso,dc=com"

Dim sTitle, sMessagePrompt, sMessageAccept, sMessageDecline
' Titlebar of all prompts
sTitle = "Microsoft Office 2007 Deployment"
' Message to display to user
sMessagePrompt = "Microsoft Office 2007 is ready to be installed "  & _
	"on your system. Installation requires a RESTART and approximately " & _
	"20 MINUTES to complete. Would you like to install now?"
' Message if user declines installation
sMessageDecline = "Thank you. You will be prompted each time you " & _
	"log on to approve the installation. If you want to launch " & _
	"the installation at a time that is appropriate for you, " & _
	"log off, log on, and approve the installation."
' Message if the user accepts installation
sMessageAccept = "Thank you. Installation will begin shortly. " & _
	VbCrLf & "IMPORTANT: After the restart, the installation will commence. " & _
	"Windows will allow you to log on during the installation. " & _
	VbCrLf & "PLEASE DO NOT LOG ON for at least 20 minutes, or until the hard disk " & _
	"activity light on your system has shown low activity for a few minutes. " & _
	"We thank you for ALLOWING THE INSTALLATION TO COMPLETE BEFORE LOGGING ON."
	
Dim sStagingGroup, sPromptGroup
' The PROMPT GROUP is the group that is causing this logon script to run.
' If the user accepts the installation, the user will no longer be prompted,
' which is done by removing the user from this group.
' Therefore, the PROMPT GROUP must have this permission assigned:
' Add/Remove Self As Member permission assigned to itself, i.e.
' CCM_Office 2007 User Staging :: ALLOW :: Add/Remove Self as Member
sPromptGroup = "CCM_Office 2007 User Staging"
' The STAGING GROUP is the group to which computers are added for the
' installation to be deployed using a startup script.
' The USER running this script must have permission to add the COMPUTER
' to the group, therefore the STAGING GROUP group must have this permission assigned:
' CCM_Office 2007 User Staging :: ALLOW :: Write :: Members
sStagingGroup = "CCM_Office 2007 Deploy" 

Dim bDebugMode, sDebugFile
' DEBUG MODE: TRUE (on) FALSE (off)
' if on, run the script then look for "C:\Software_Deploy_Log.txt"
bDebugMode = False
' Filename to dump results to.  Set to ECHO to dump results to console.
' This runs in the user's context and users cannot write files to c:\
' Be certain users can write to this location
sDebugFile = "ECHO"

' END CONFIGURATION BLOCK
'==========================================================================

Dim retVal
retVal = WriteFile (sDebugFile, "SCRIPT RUNS", True)

' Popup constants
Const popOK 		= 0
Const popOKCancel	= 1
Const popYesNo 		= 4
Const popStop		= 16
Const popQuestion	= 32
Const popExclam		= 64
Const popInfo 		= 128
Const btnOK			= 1
Const btnCancel		= 2
Const btnYes 		= 6
Const btnNo 		= 7

' Initialization
Dim sComputerName
sComputerName = GetComputerName()
Dim WSHShell
Set WSHShell = CreateObject("WScript.Shell")
Dim FSO
Set FSO = CreateObject("Scripting.FileSystemObject")
Dim ADSys, sUserDN, oUser, sUsername
Set ADSys = CreateObject("ADSystemInfo")
sUserDN = ADSys.UserName
Set oUser = GetObject("LDAP://" & sUserDN)
sUsername = oUser.samAccountName

Dim sScriptResults
Dim bRestart
bRestart = False

Dim sMessage
' Prompt user
sMessage = sMessagePrompt
retVal = WSHShell.Popup (sMessage, 0, sTitle, popYesNo + popQuestion)

If retVal = btnYes Then
	' Installation accepted
	sScriptResults = sScriptResults & "Installation accepted" & VbCrLf
	sMessage = sMessageAccept
	retVal = WSHShell.Popup (sMessage, 0, sTitle, popOKCancel + popExclam)
	If retVal = btnOK Then
		' Installation confirmed
		sScriptResults = sScriptResults & "Installation confirmed" & VbCrLf
		' Add computer to staging group
		If Group_AddMember(sStagingGroup, sComputerName) Then 
			sScriptResults = sScriptResults & sComputerName & " added to " & sStagingGroup & VbCrLf
			bRestart = True
			' remove user from prompt group
			If Group_RemoveMember(sPromptGroup, sUsername) Then
				sScriptResults = sScriptResults & sUsername & " removed from " & sPromptGroup & VbCrLf
			Else
				sScriptResults = sScriptResults & "ERROR: Could not remove " & sUsername & " from " & sPromptGroup & VbCrLf
			End If
		End If
	Else
		sScriptResults = sScriptResults & "Installation cancelled" & VbCrLf
	End If
Else
	sScriptResults = sScriptResults & "Installation declined" & VbCrLf
End If

' -------------------------------------------------------------------------
' Wrap up the script

' DEBUG
If bDebugMode Then
	If ucase(sDebugFile) = "ECHO" Then
		MsgBox "Please notify your administrator of the following results:" & VbCrLf & _
			sScriptResults
	Else
		retVal = WriteFile(sDebugFile, sScriptResults, True)
	End If
End If

If bRestart Then
	If bDebugMode Then
		MsgBox "Restart cancelled due to debug mode"
	Else
		retVal = Computer_Reboot(".")
	End If
Else
	WScript.Quit(0)
End If

Function Execute_Capture(ByVal sCommand, ByVal iTimeout, ByVal bTerminate)
	' VERSION 071122
	' Executes command in sCommand
	' Times out after iTimeout and (optionally) terminates the process
	' Returns exit code, StdOut & StdErr
	'
	' USAGE:	arrayvariable = Execute_Capture(sCommand, iTimeout, [true|false])
	'
	' INPUTS:	sCommand: 			Command to run
	'			iTimeout:			Time (in seconds) to wait for process
	'			bTerminate:			TRUE  = terminate process after timeout
	'								FALSE = return from function
	' RETURNS:	Execute_Capture:	*ARRAY*
	'				(0): 			Exit code of command 
	'								0.01 = timed out but NOT terminated
	'								0.99 = timed out and TERMINATED
	'				(1):			StdOut
	'				(2):			StdErr
	'
	' REQUIRES:	global definition of WSHShell
	'			or
	'			uncomment next line:
	' 			Dim WSHShell : Set WSHShell = CreateObject("WScript.Shell")
	
	Dim oExec, iTimer, iExitCode, sOut, sErr
	
	On Error Resume Next
	sScriptResults = sScriptResults & sCommand & VbCrLf 
	Set oExec = WSHShell.Exec(sCommand)
	If Err.Number <> 0 Then 
		' There was a problem running the command itself
		iExitCode = Err.Number
		sStdOut = ""
		sStdErr = "Failed to run command " & sCommand & VbCrLf & Err.Description
		Execute_Capture = Array(iExitCode, sStdOut, sStdErr)
		Err.Clear
		Exit Function
	End if
	
	' Command was launched. Wait for completion or timeout.
	On Error Goto 0
	iTimer = 0
	Do While oExec.Status = 0
		WScript.Sleep 100
		iTimer = iTimer + 100
		If (iTimer/1000) > iTimeout Then Exit Do
	Loop
	
	If oExec.Status = 0 Then
		' Not completed, but timed out
		If bTerminate Then
			' Terminate
			oExec.Terminate
			iExitCode = 0.99
		Else
			' Timeout but do not terminate
			iExitCode = 0.01
		End If
	Else
		' Completed
		iExitCode = oExec.ExitCode
	End If

	sOut = oExec.StdOut.ReadAll
	sErr = oExec.StdErr.ReadAll
	Execute_Capture = Array(iExitCode, sOut, sErr)
	
End Function

' ==========================================================================
' GROUP MEMBERSHIP ROUTINES
' The following routines manipulate group membership

Function Group_AddMember (ByVal sGroupName, ByVal sMemberName)
	' Adds a member to a group
	' Inputs:	sGroupName	 	samAccountName of the group
	'			sMemberName 	samAccountName (user/group) or Name (computer) of member to add
	' Requires:	sDomainDN		the DN of the domain
	'			sScriptResults	global declaration - accumulates script results
	' 			ErrorHandler	function
	
	Dim sGroupADsPath, oGroup, sMemberADsPath, oMember
	sGroupADsPath = ADObject_Find_UCG(sGroupName, sDomainDN)
	If sGroupADsPath = "" Then
		Group_AddMember = False
		sScriptResults = sScriptResults & ErrorHandler("Group_AddMember", "Could not find group: " & sGroupName)
		Exit Function
	End If
	
	On Error Resume Next
	Set oGroup = GetObject(sGroupADsPath)
	If Err.Number <> vbEmpty Then
		Group_AddMember = False
		sScriptResults = sScriptResults & ErrorHandler ("Group_AddMember", "Could not connect to group: " & sGroupName)
		Err.Clear
		Exit Function
	End If
	
	sMemberADsPath = ADObject_Find_UCG(sMemberName, sDomainDN)
	If sMemberADsPath = "" Then
		Group_AddMember = False
		sScriptResults = sScriptResults & ErrorHandler ("Group_AddMember", "Could not find: " & sMemberName)
		Exit Function
	End If
	
	oGroup.Add(sMemberADsPath)
	If Err.number <> vbEmpty Then
		Group_AddMember = False
		sScriptResults = sScriptResults & _
			ErrorHandler ("Group_AddMember", "Could not add: " & sMemberName & " (" & sMemberADsPath & ") " & _
			" to group " & sGroupName & " (" & sGroupADsPath & ")")
		Exit Function
	End If
	Group_AddMember = True
	
End Function

Function Group_RemoveMember (ByVal sGroupName, ByVal sMemberName)
	' Removes a member from a group
	' Inputs:	sGroupName		samAccountName of the group
	' 			sMemberName 	samAccountName (user/group) or Name (computer) of member to add
	' Requires:	sDomainDN 		the DN of the domain
	'			sScriptResults	global declaration - accumulates script results
	' 			ErrorHandler 	function
	
	Dim sGroupADsPath, oGroup, sMemberADsPath, oMember
	sGroupADsPath = ADObject_Find_UCG(sGroupName, sDomainDN)
	If sGroupADsPath = "" Then
		Group_RemoveMember = False
		sScriptResults = sScriptResults & ErrorHandler("Group_RemoveMember", "Could not find group: " & sGroupName)
		Exit Function
	End If
	
	On Error Resume Next
	Set oGroup = GetObject(sGroupADsPath)
	If Err.Number <> vbEmpty Then
		Group_RemoveMember = False
		sScriptResults = sScriptResults & ErrorHandler ("Group_RemoveMember", "Could not connect to group: " & sGroupName)
		Err.Clear
		Exit Function
	End If
	
	sMemberADsPath = ADObject_Find_UCG(sMemberName, sDomainDN)
	If sMemberADsPath = "" Then
		Group_RemoveMember = False
		sScriptResults = sScriptResults & ErrorHandler ("Group_RemoveMember", "Could not find: " & sMemberName)
		Exit Function
	End If
	
	oGroup.Remove(sMemberADsPath)
	If Err.number <> vbEmpty Then
		Group_RemoveMember = False
		sScriptResults = sScriptResults & _
			ErrorHandler ("Group_RemoveMember", "Could not remove " & sMemberName & " (" & sMemberADsPath & ") " & _
			" from group " & sGroupName & " (" & sGroupADsPath & ")")
		Exit Function
	End If
	Group_RemoveMember = True 
	
End Function

Function ADObject_Find_UCG(sObjectName, sSearchDN)
	' VERSION 070706
	' Returns the full ADsPath (LDAP://...) of a user, computer, or group
	' Inputs:
	' 		sObjectName: The unique identifier for the object class.  The script supports:
	'			User: 		sAMAccountName (pre-Windows 2000 logon name)
	'			Group: 		sAMAccountName (pre-Windows 2000 logon name)
	'			Computer: 	Name (translated by the script to the computer's sAMAccountName by adding a $)
	' 		sSearchDN: the DN within which to search (often, the DN of the domain, e.g. dc=contoso, dc=com)

	Dim oConnection
	Dim oRecordset
	Dim sLDAPObjectQuery
	Dim sLDAPIdentifierQuery
	Dim sLDAPQuery
	Dim sProperties
	Dim oADObject
	Dim aProperties
	Dim sProperty
	Dim sLDAPIdentifier
	Dim sSearchScope
	
	sLDAPIdentifier = "samAccountName"
	sProperties = "ADsPath"
	sSearchScope = "subtree"
	
	' Open an ADO connection using null credentials
	Set oConnection = CreateObject("ADODB.Connection")
	oConnection.Provider = "ADsDSOObject"
	'On Error Resume Next
	oConnection.Open "", vbNullString, vbNullString
	If oConnection.State = 0 Then ' 0 = adStateClosed
	     ' Error handling code: can't connect to AD
	     ADObject_Find_UCG = ""
	     Exit Function
	End If
	
	' Build the LDAP Query
	sLDAPQuery = "<LDAP://" & sSearchDN & ">;"
	sLDAPQuery = sLDAPQuery & _
		"(|(samAccountName=" & sObjectName & ")" & _
		"(samAccountName=" & sObjectName & "$));" & _
		sProperties & ";" & sSearchScope
	
	' Retrieve the result set, close the connection, and check to make
	' sure we received at least one result
	Set oRecordset = oConnection.Execute (sLDAPQuery)
	If oRecordset.EOF and oRecordset.BOF Then
		' Error handling code: no object found
	    ADObject_Find_UCG = ""
	     Exit Function
	End If
	
	ADObject_Find_UCG = oRecordset.Fields("ADsPath")
	oRecordset.Close
	oConnection.Close
	
End Function

' ==========================================================================
' MISCELLANEOUS

Function ErrorHandler( ByVal sRoutine, ByVal sErrorText )
	' Returns a string describing the error and clears Err
    Dim sErrorHandler
    sErrorHandler = String( 50, "-") & VbCrLf & _
    	"Error in: " & sRoutine & VbCrLf & _
    	sErrorText & VbCrLf & _
		"Error number: " & Err.number & VbCrLf & _
    	"Error Description: " & Err.Description & vbCrLf
    Err.Clear
    ErrorHandler = sErrorHandler  
End Function

Function GetComputerName()
    Dim WshNetwork
    Set WshNetwork = WScript.CreateObject("WScript.Network")
    GetComputerName = WshNetwork.ComputerName 
End Function

Function WriteFile(ByVal FilePath, ByVal Text, ByVal Overwrite)
	WriteFile = False
	
	Const ForWriting = 2, ForAppending = 8, TristateUseDefault = -2
	Dim FSO, oFile, IOMode
	
	If Overwrite Then 
		IOMode = ForWriting
	Else
		IOMode = ForAppending
	End If
	
	Set FSO = CreateObject("Scripting.FileSystemObject")
	If FSO.FolderExists( FSO.GetParentFolderName(FilePath) ) Then
		On Error Resume Next
		Set oFile = FSO.OpenTextFile(FilePath, IOMode, _
			True, TristateUseDefault)
		oFile.Write Text
		oFile.Close
		If Err then
			Err.Clear
			WriteFile = False
		Else
			WriteFile = True
		End if
	End If
	
End Function

Function Computer_Reboot(sComputer)
	' Restart computer
	' Version 071122
	
	Dim oWMI, cOperatingSystems, oOperatingSystem
	Dim ReturnValue
	
	Set oWMI = GetObject("winmgmts:{impersonationLevel=impersonate,(Shutdown)}!\\" & _
		sComputer & "\root\cimv2")
	Set cOperatingSystems = oWMI.ExecQuery _
		("Select * from Win32_OperatingSystem")
	For Each oOperatingSystem in cOperatingSystems
		oOperatingSystem.Reboot()
	Next
End Function
